package edu.udo.cs.URLChecker;

import java.net.*;
import java.io.*;
import java.util.*;

import java.util.regex.*;
import sun.net.www.protocol.http.HttpURLConnection;

/**
 * Diese Klasse dient der Suche in HTML-Seiten nach Links und verfolgt diese rekusiv weiter
 * bis zu einer gewissen Tiefe
 * 
 * @author hakenjos
 * @version 1.0
 *
 */
public class Checker {

	private String url;
//	private String error_protocoll;
	private StringBuffer error_protocoll;
	private boolean rekursiv,extended, protocoll,doc_analysis,matching;
	private String p_file,doc_analysis_file,matching_file;
	private int depth, actualdepth, allpages, errors;
	private boolean ready = false, isFrame = false, stop = false;
	private URL header;
	private String cookieInternal;
	private String user,passwd;
	
	private OutputStream docstream;

	private boolean internal = true;
	
	private Collection patterns;

	/**
	 * Constructor for Checker.
	 */
	public Checker(String url, boolean rekursiv, boolean internal, int depth,boolean matching, String matching_file,boolean doc_analysis, String doc_analysis_file, boolean extended, boolean protocoll, String p_file) {
		this.url = url;
		this.rekursiv = rekursiv;
		this.internal = internal;
		this.depth = depth;
		this.matching=matching;
		this.matching_file=matching_file;
		this.doc_analysis=doc_analysis;
		this.doc_analysis_file=doc_analysis_file;
		this.extended=extended;
		this.protocoll=protocoll;
		this.p_file=p_file;
		this.actualdepth = 0;
		this.allpages = 0;
		this.error_protocoll = new StringBuffer();
		this.errors = 0;
		this.user="";
		this.passwd="";
		try {
			this.header = new URL(url);
		} catch (Exception e) {
			System.out.println("Init header: " + e.getMessage());
		}
		initDocumenting();
		initMatching();
	}
	
	public void initMatching(){
		if (matching){
			try{
				patterns=new Vector();
				LineNumberReader reader=new LineNumberReader(new FileReader(new File(matching_file)));
				String line=reader.readLine();
				Pattern pattern;
				while (line!=null){
					pattern=Pattern.compile(line);
					patterns.add(pattern);
					line=reader.readLine();
				}
			}catch(Exception e){
				System.err.println(e.getMessage());
				matching=false;
			}
		}
	}
	
	public void initDocumenting(){
		docstream=System.out;
		if (doc_analysis){
			try{
				docstream=new FileOutputStream(new File(doc_analysis_file));
			}catch(Exception e){
				System.err.println(e.getMessage());
				docstream=System.out;
			}
		}
	}

	public int getErrors() {
		return errors;
	}

	/**
	 * Maximale Suchtiefe
	 * 
	 */
	public int getDepth() {
		return this.depth;
	}

	/**
	 * Aktuelle Suchtiefe
	 * 
	 */
	public int getActualDepth() {
		return this.actualdepth;
	}

	/**
	 * count analyzed Sites
	 */
	public int countAnalyzedSites() {
		return this.allpages;
	}

	/** R?ckkehr zur url-Port-Nummer; wenn diese nicht definiert ist,
	 * wird die Default-HTTP-Port-Nummer gew?hlt
	 */
	int getPort(URL url) {
		int p = url.getPort();
		if (p == -1) {
			p = 80;
		}
		return p;
	}

	/** Die folgende Methode h?ngt die m?glicherweise fehlende Directory-Angabe
	 * einer URL an.
	 * 
	 */
	static String adjustIfDir(String s) {
		int p = s.lastIndexOf("/") + 1;
		if (!s.endsWith("/") && s.indexOf(".", p) < 0) {
			s += "/";
		}
		return s;
	}

	public String analyzeTag(String sub, URL url) {
		StringTokenizer st = new StringTokenizer(sub, " ");
		String nextToken;

		// es ist ein EndTag
		if (sub.startsWith("/"))
			return null;

		//Ist es ein Link?
		int alink = Math.max(sub.indexOf("a "), sub.indexOf("A "));
		int href = -1;
		if (alink >= 0) {
			href = Math.max(sub.indexOf("href="), sub.indexOf("HREF="));
		}
		if ((alink >= 0) && (href >= 0)) {
			//yes it's a link
			sub = sub.substring(href + 5);
			st = new StringTokenizer(sub, String.valueOf('"'));
			if (!st.hasMoreTokens()) {
				return null;
			}
			return st.nextToken();
		} //Link jetzt erfolgreich gefunden
		//Links auch in Image-Map
		alink = Math.max(sub.indexOf("area "), sub.indexOf("AREA "));
		href = Math.max(sub.indexOf("href="), sub.indexOf("HREF="));
		if ((alink >= 0) && (href >= 0)) {
			//yes it's a link
			sub = sub.substring(href + 5);
			st = new StringTokenizer(sub, String.valueOf('"'));
			if (!st.hasMoreTokens()) {
				return null;
			}
			return st.nextToken();
		} //Link jetzt erfolgreich gefunden
		//Frames wie Link behandeln, jedoch frame-Flag setzen, damit behandelbar
		int frame = Math.max(sub.indexOf("frame "), sub.indexOf("FRAME "));
		int src = Math.max(sub.indexOf("src="), sub.indexOf("SRC="));
		if ((frame >= 0) && (src >= 0)) {
			sub = sub.substring(src + 4);
			st = new StringTokenizer(sub, String.valueOf('"'));
			if (!st.hasMoreTokens()) {
				return null;
			}
			isFrame = true;
			return st.nextToken();

		}

		return "";
	}

	public void check() {
		this.ready = false;
		if (internal){
			connectToInternal();
		}
		walk(header);
		ready = true;
	}

	//diese Methode zum Durchlaufen einer Seite benutzen, ist nicht rekursiv
	public void walk(URL head) {

		LinkedList walkLinks = new LinkedList();
		Vector analyzedSites = new Vector();

		Vector v = new Vector();
		try {
			v =
				findTags(
					new WebSite(
						null,
						new URL(adjustIfDir(head.toString())),
						0,
						0));
		} catch (Exception e) {
			printError(
				"Error during findTags in URL: "
					+ head.toExternalForm()
					+ "\n"
					+ e.getMessage());
		}
		allpages++;
		for (int i = 0; i < v.size(); i++) {
//			walkLinks.add((WebSite) v.get(i));
			addWebSite(walkLinks,(WebSite) v.get(i));
		}

		WebSite actual;

		while ((walkLinks.size() > 0) && (!stop)) {
			//ersten Eintrag nehmen

			actual = (WebSite) walkLinks.removeFirst();

			v = new Vector();

			if ((actual.getDepth() <= depth)
				&& (!analyzedSites.contains(actual.getURL().toString()))) {
				v = findTags(actual);
/*				if (actual.getURL().toString().equals("http://www-ai.cs.uni-dortmund.de/ls8/index.html")){
					for (int i = 0; i < v.size(); i++) {
						System.out.println("gefundener Link: "+v.get(i));
					}
				}*/
				analyzedSites.add(actual.getURL().toString());
				allpages++;
			}

			// Jetzt werden alle gefundenen Links dem Vector hinzugefuegt

			for (int i = 0; i < v.size(); i++) {
//				walkLinks.add((WebSite) v.get(i));
				addWebSite(walkLinks,(WebSite) v.get(i));
			}
		} //endwhile
		
		OutputStream out=docstream;

		if (protocoll){
			try{
				File file=new File(p_file);
				out=new FileOutputStream(file);
			}catch(Exception e){
				System.err.println(e.getMessage());
				out=docstream;
			}
		}
		else{
			out=docstream;
		}
		
		try{
			out.write("Error-Protocol:\n".getBytes());
			out.write(getErrorProtocol().getBytes());
			out.write("\n\n".getBytes());
			out.write(("Number of analyzed sites: " + allpages+"\n").getBytes());
			out.write(("Number of Errors: " + errors+"\n").getBytes());
			out.write("\n".getBytes());
		}catch(Exception e){
			System.err.println("Can't write the Error-Protocoll!");
		}
	}


	/**
	 * WebSites auf dessen URL ein Pattern passt, werden nicht weiterverfolgt;
	 */	
	private void addWebSite(LinkedList walklinks,WebSite website){

		if (!this.matching){
			walklinks.add(website);
			return;
		}

		String url=website.getURL().toString();
		boolean m=false;
		Iterator iter=patterns.iterator();
		Pattern pattern;
		Matcher matcher;
		while (iter.hasNext()){
			pattern=(Pattern) iter.next();
			matcher=pattern.matcher(url);
			m=matcher.matches();
		}
		
		if (!m){
			walklinks.add(website);
		}		
		
	}

	// Suche aller Links in einer url und R?ckgabe in einen Vektor
	//Vektor enth?lt WebSites
	//Es kann natuelich erst beim durchlaufen der Seite festgestellt werden, ob es eine Fehlerseite ist, 
	//deshlab wird in dieser Methode eine Fehlerbehandlung vorgenommen.
	//Es wird ?berpr?ft, ob es sich um eine Fehlerzeile handelt.
	public Vector findTags(WebSite seite) {
		/*		if (seite.siteURL.toString().indexOf("http://www-ai.cs.uni-dortmund.de/index3.html")!=-1){
					System.out.println("Verfolge jetzt Seite: http://www-ai.cs.uni-dortmund.de/index3.html");
				}
				if (seite.siteURL.toString().indexOf("http://www-ai.cs.uni-dortmund.de/LEHR/")!=-1){
					System.out.println("Verfolge jetzt Seite: http://www-ai.cs.uni-dortmund.de/LEHR/");
				}
		*/
		
		String indent=new String("");
		for (int i = 0; i < seite.getDepth(); i++) {
			indent+="\t";
//			System.out.print(i + "\t");
		}
		indent+=seite.getDepth()+"\t";

		printInfo(indent+
			"analyzing: " + seite.getURL().toString() + "....................");

		URL url = null;
		try {
			url = new URL(seite.getPredecessor(), seite.getURL().toString());
		} catch (Exception e) {
			printError("URL " + e.getMessage());
		}

		Vector v = new Vector();
		//connectToInternal();
		HttpURLConnection con = null;
		try {
			con = (HttpURLConnection) url.openConnection();
			con.setRequestMethod( "GET");
		} catch (Exception e) {
			printError("con " + e.getMessage());
		}

		if (internal) {
			con.setRequestProperty("Cookie", cookieInternal);
		}

		//Damit nicht ein evtl. Cache zum Laden der Seite verwendet wird
		con.setUseCaches(false);


		WebSite site;

		try {
			if (con != null) {
				BufferedReader in =
					new BufferedReader(new InputStreamReader(con.getInputStream()));
				String zeile;
				boolean specialFile = isSpecialFile(seite);
				int line = 0;
				int maxline = 10000;
				if (specialFile) {
					maxline = 100;
				}

				while (((zeile = in.readLine()) != null)
					&& (!stop)
					&& (line <= maxline)) {
					line++;
//					seite.setLine(line);
					while (zeile != null) {
						//System.out.println(zeile);
						//Taganfang suchen

						//Ist dies eine Fehlerseite
						if (isErrorLine(zeile)) {
							//This is a real error
							addError(
								"analyzing: "
									+ seite.getURL().toString()
									+ "....................\n"+seite.toString() + "\n");
							printError(seite.toString());
							return new Vector();

						}
						int p = zeile.indexOf("<");
						if (p < 0) {
							break;
						}

						// Kontrolle, ob sich '>' in derselben Zeile befindet
						int q = 0;
						while ((q = zeile.indexOf(">", p)) < 0) {
							String m = in.readLine();
							if (m == null) { // EOF erreicht
								return v;
							}
							zeile += m;
						}
						String sub = zeile.substring(p + 1, q);

						//System.out.println("analysiere Tags "+sub);

						String u = null;
						if (!seite.isExternal())
							u=analyzeTag(sub, url);

						if (u!=null){
							int rauteindex=u.indexOf("#");
							if (rauteindex>0){
								u=u.substring(0,rauteindex);
							}
							/*
							while (u.startsWith("../")){
								u = u.substring(3);
							}*/
							if ((u.indexOf("javadoc")>=0)||(u.indexOf("apidoc")>=0)){
								u=null;
							}
						}
						if ((u != null) && (u.startsWith("../"))) {
//							u = u.substring(3);
						}
						if ((u != null && u.length() > 0)
							&& (u.indexOf("mailto:") == -1)
							&& (!isFrame)) {
							URL n = new URL(url, adjustIfDir(u));
							if (n.getHost().equals(url.getHost())){
								v.addElement(
									new WebSite(
										url,
										new URL(url, adjustIfDir(u)),
										line,
										seite.getDepth() + 1));
							}
							else{
								//ein externer Link
								v.addElement(
									new WebSite(
										url,
										new URL(u),
										line,
										seite.getDepth() + 1,true));
							}
							//System.out.println("gefundener Link:"+line);
						}
						if ((u != null) && (isFrame)) {
							isFrame = false;
							Vector v1 =
								findTags(
									new WebSite(
										url,
										new URL(url, adjustIfDir(u)),
										line,
										seite.getDepth() + 1));
							Enumeration enum = v1.elements();
							while (enum.hasMoreElements()) {
								v.add((WebSite) enum.nextElement());
							}
							isFrame = false;
						}

						// Fortsetzung der Tag-Suche
						zeile = zeile.substring(q + 1);
					}
				}
				in.close();
				in = null;
			}

		} catch (Exception e) {
			//errors++;
			addError(
				"analyzing: "
					+ seite.getURL().toString()
					+ "....................\n"+"findTags Ende " + e.getMessage() + "\n\n");
			printError("findTags Ende " + e.getMessage());

		}
		return v;
	}

	public boolean isErrorLine(String error) {

		if (error.indexOf("File (or Template) not found:") != -1) {
			return true;
		}

		return false;
	}

	public boolean isSpecialFile(WebSite seite) {
		String urlstring = seite.getURL().toString();

		if (urlstring.endsWith(".pdf"))
			return true;
		if (urlstring.endsWith(".zip"))
			return true;
		if (urlstring.endsWith(".jar"))
			return true;
		if (urlstring.endsWith(".gz"))
			return true;
		if (urlstring.endsWith(".ppt"))
			return true;
		if (urlstring.endsWith(".ps"))
			return true;
		if (urlstring.endsWith(".pps"))
			return true;
		if (urlstring.endsWith(".djvu"))
			return true;
		return false;
	}
	
	public void printInfo(String info){
		if (this.extended){
			try{
				docstream.write((info+"\n").getBytes());
			}catch(Exception e){
				System.err.println(e.getMessage());
			}
		}
	}

	public void printError(String error){
		try{
			docstream.write((error+"\n").getBytes());
		}catch(Exception e){
			System.err.println(e.getMessage());
		}
	}

	public void addError(String error) {
		if (error.indexOf("javascript") < 0) {
			//nur Fehler mitaufnehmen,
			//die keine javascript-Fehler sind
			this.error_protocoll.append(error + "\n");
			errors++;
		}
	}

	public String getErrorProtocol() {
		return this.error_protocoll.toString();
	}

	public void connectToInternal() {
		if (internal) {
			try {
				
				URL url2 = new URL("http://kissen.cs.uni-dortmund.de:8080/post-login.html");
				HttpURLConnection connection = (HttpURLConnection) url2.openConnection();
				connection.setDoOutput(true);
				connection.setInstanceFollowRedirects(false);
  		    	connection.setRequestMethod( "POST");

				OutputStreamWriter w = new OutputStreamWriter(connection.getOutputStream());
				w.write("password="+passwd+"&login="+user);
				w.close();

			
				//System.out.println("Response code:" + connection.getResponseCode());			
				String cookieInternalRaw = connection.getHeaderField("Set-Cookie");
				this.cookieInternal = cookieInternalRaw.substring(0,
				cookieInternalRaw.indexOf(";"));
				//System.out.println("Das ist das Cookie:" + cookieInternalRaw + "#");
				
				//dump(connection);

/*	
			url2 = new URL("http://kissen.cs.uni-dortmund.de:8080/auto?expr=DuDSeminar");
			connection = (HttpURLConnection) url2.openConnection();
			connection.setRequestMethod( "GET");
			connection.setRequestProperty("Cookie", cookieInternal);

			System.out.println("#" + connection.getRequestProperty("Cookie") + "#");
			
			connection.setUseCaches(false);
			System.out.println(connection.toString());	
			dump(connection);

*/

			} catch (Exception e) {
				printError("connection " + e.getMessage());
			}
		}
	}

	public static void dump(URLConnection connection) throws IOException {

		int n = 1;
		// n=0 has no key, and the HTTP return status in the value field
		boolean done = false;
		while (!done) {
			String headerKey = connection.getHeaderFieldKey(n);
			String headerVal = connection.getHeaderField(n);
			if (headerKey != null || headerVal != null) {
				System.out.println(headerKey + "=" + headerVal);
			} else {
				done = true;
			}
			n++;
		}

		BufferedReader in =
			new BufferedReader(
				new InputStreamReader(connection.getInputStream()));
		System.out.println("inputLine");

		String inputLine = null;
		while ((inputLine = in.readLine()) != null)
			System.out.println(inputLine);

		in.close();
	}

	public void setUser(String user){
		this.user=user;
	}
	
	public void setPassword(String passwd){
		this.passwd=passwd;
	}

/*	public static void main(String args[]) {
		Checker ch = new Checker("http://www-ai", false, true, 2);
		ch.connectToInternal();
	}*/

}
