package edu.udo.cs.URLChecker;

import java.io.LineNumberReader;
import java.io.File;
import java.io.FileReader;
/**
 * Main-Class.
 * @author hakenjos
 * @version 1.0
 */
public class URLChecker {

	private String startURL;
	private boolean rekursiv;
	private boolean internal;
	private int depth;
	private boolean extended, protocoll,doc_analysis,matching;
	private String p_file,doc_analysis_file,loginfile,matching_file;

	/**
	 * Constructor for URLChecker.
	 */
	public URLChecker(
		String startURL,
		boolean rekursiv,
		boolean internal,
		String loginfile,
		boolean matching,
		String matching_file,
		int depth,
		boolean doc_analysis,
		String doc_analysis_file,
		boolean extended,
		boolean protocoll,
		String p_file) {
			
			
		this.startURL = startURL;
		this.rekursiv = rekursiv;
		this.internal = internal;
		this.loginfile=loginfile;
		this.matching=matching;
		this.matching_file=matching_file;
		this.depth = depth;
		this.doc_analysis=doc_analysis;
		this.doc_analysis_file=doc_analysis_file;
		this.extended=extended;
		this.protocoll=protocoll;
		this.p_file=p_file;
	}

	public void check() {
		Checker ch = new Checker(startURL, rekursiv, internal, depth,matching,matching_file,doc_analysis, doc_analysis_file, extended,protocoll,p_file);

		if (internal) {
			String user=null,passwd=null;
			
			if (loginfile==null){
				LoginFrame login = new LoginFrame("Login required");
				user = login.getUser();
				passwd = login.getPassword();

			}
			else{
				try{
					LineNumberReader reader=new LineNumberReader(new FileReader(new File(loginfile)));
					user=reader.readLine();
					passwd=reader.readLine();
				}catch(Exception e){
					System.err.println(e.getMessage());
					user=null;
					passwd=null;
				}
			}

			if ((user == null) || (passwd == null)) {
				System.out.println("Username and password are required!");
				System.exit(0);
			}

			ch.setUser(user);
			ch.setPassword(passwd);
		}
		
		ch.check();

	}

	public static void main(String[] args) {

		if (args.length == 0) {
			System.out.println(
				"\nUsage for LinkChecking:\n\n\tjava -jar URLChecker.jar [-r <depth>] [-i [<file>]] [-m <file>] [-d <file>] [-x] [-p <file>] -url <url>");
			System.out.println(
				"\n\nFor Help:\n\n\tjava -jar URLChecker.jar -h");
			System.exit(0);
		}

		if (args[0].equals("-h")) {
			System.out.println(
				"\nUsage for LinkChecking:\n\n\tjava -jar URLChecker.jar [-r <depth>] [-i [<file>]] [-m <file>] [-d <file>] [-x] [-p <file>] -url <url>\n\n\n");
			System.out.println(" -r <depth>\t Rekursive Suche bis zu der angegebenen Tiefe\n");
			System.out.println(" -i [<file>]\t Analysiere auch den internen Bereich.\n\t\t Die Login-Information sind in der angebenen Datei.\n\t\t Inder ersten Zeile sollte der Name des Users stehen und in der Zweiten das Passwort.\n");
			System.out.println(" -m <file> \t Die Datei enthaelt zeilenweise regulaere Ausdruecke.\n\t\t Sollte ein Matching mit einer URL erfolgreich sein,\n\t\t so wird sie nicht analysiert.\n");
			System.out.println(" -d <file> \t Dokumentiere die Analyse in dieser Datei,\n\t\t ansonsten Ausgabe in der Konsole.\n");
			System.out.println(" -x        \t In diesem erweitertem Dokumentierungsmodus wird auch\n\t\tdie aktuell untersuchte Seite ausgegeben.\n");
			System.out.println(" -p <file> \t Schreibe das Fehlerprotokoll in diese Datei.\n\t\t Bei Nichtangabe erfolgt die Ausgabe ueber die angegebene Dokumentationsausgabe.\n"); 
			System.out.println("\n");
			System.out.println(" -url <url>     \t Start-URL.\n");
			System.exit(0);

		}

		int depth = 0;
		boolean rekursiv = false,
			questiontask = false,
			extended = false,
			protocoll = false,
			url_found = false,
			doc_analysis=false,
			matching=false;
		boolean intern = false;

		String url = "", logfile = "", protocollfile = "",doc_analysis_file="",login_file=null,matchingfile="";

		int args_length = args.length;

		boolean error_args = false;

		try {

			boolean r_found = false,
				i_found = false,
				x_found = false,
				p_found = false,
				d_found= false,
				m_found=false;
			url_found=false;
			int index = 0;
			while (index < args_length) {
				if ((args[index].equals("-r")) && (!r_found)) {
					rekursiv = true;
					index++;
					depth = Integer.parseInt(args[index]);
					r_found = true;
				}
				if ((args[index].equals("-i")) && (!i_found)) {
					i_found = true;
					intern = true;
					if (!args[index+1].startsWith("-")){
						//d.h. loginfile ist angegeben.
						index++;
						login_file=args[index];
					}
				}
				if ((args[index].equals("-x")) && (!x_found)) {
					x_found = true;
					extended = true;
				}
				if ((args[index].equals("-p")) && (!p_found)) {
					p_found = true;
					protocoll = true;
					index++;
					protocollfile = args[index];

				}
				if ((args[index].equals("-d")) && (!d_found)) {
					d_found = true;
					doc_analysis = true;
					index++;
					doc_analysis_file=args[index];

				}
				if ((args[index].equals("-m")) && (!m_found)) {
					m_found = true;
					matching = true;
					index++;
					matchingfile=args[index];
				}


				if ((args[index].equals("-url")) && (!url_found)) {
					url_found = true;
					index++;
					url = args[index];
				}

				index++;
			}

			if (!i_found) {
				intern = false;
			}
			if (!x_found) {
				extended = false;
			}
			if (!url_found) {
				error_args = true;
			}
			if (!d_found){
				doc_analysis=false;
			}

		} catch (Exception error) {
			error_args = true;
		}

		if (!error_args) {
			System.out.println(
				"-----------------------Begin analyzing!-------------------------");

			URLChecker urlc = new URLChecker(url, rekursiv, intern,login_file,matching,matchingfile, depth,doc_analysis,doc_analysis_file, extended, protocoll, protocollfile);
			urlc.check();
			System.out.println(
				"-----------------------End analyzing!---------------------------");
			System.exit(0);

		} else {

			//falsche Parameterl?nge etc
			System.out.println(
				"\nUsage for URLChecking:\n\n\tjava -jar URLChecker.jar [-r <depth>] [-i [<file>]] [-m <file>] [-d <file>] [-x] [-p <file>] -url <url>");
			System.out.println(
				"\n\nFor Help:\n\n\tjava -jar URLChecker.jar -h");
			System.exit(0);

		}

	}

}
